#include "MgOpenSSLPluginI.h"
#include <Ice/LoggerUtil.h>
using Ice::Error;
using Ice::Trace;
#include <Ice/Properties.h>
#include <Ice/ProtocolPluginFacade.h>
#include <Ice/Communicator.h>
#include <Ice/Exception.h>
using Ice::Exception;
#include <IceSSL/SslEndpoint.h>
using IceSSL::SslEndpointFactory;
#include <IceSSL/TraceLevels.h>

#include <iostream>
using std::cout;
using std::cerr;
using std::endl;

using std::string;
using Ice::Plugin;
using Ice::CommunicatorPtr;
using Ice::StringSeq;
using IceSSL::OpenSSLPluginI;
using IceSSL::Client;
using IceSSL::Server;
using IceSSL::ClientServer;
using IceSSL::CertificateFile;
using IceSSL::CertificateDesc;
using IceSSL::DiffieHellmanParamsFile;

//---------------------------------------------------------------------
//
// Plugin factory function
//
extern "C"
ICE_SSL_API Ice::Plugin*
create(const CommunicatorPtr& communicator,
       const string& name, const StringSeq& args)
{
    IceInternal::ProtocolPluginFacadePtr facade = IceInternal::getProtocolPluginFacade(communicator);

    OpenSSLPluginI* plugin = new MgIceSSL::MgOpenSSLPluginI(facade);
    try
    {
        plugin->configure();

        //
        // Install the SSL endpoint factory
        //
        IceInternal::EndpointFactoryPtr sslEndpointFactory = new SslEndpointFactory(plugin);
        facade->addEndpointFactory(sslEndpointFactory);
    }
    catch(const Exception& ex)
    {
        Ice::PluginPtr ptr = plugin; // Reclaim the plug-in instance

        Error out(communicator->getLogger());
        out << "exception in MgIceSSL plug-in:\n" << ex;

        // Can't throw from an extern "C" function
        return 0;
    }
    catch(...)
    {
        Ice::PluginPtr ptr = plugin; // Reclaim the plug-in instance

        Error out(communicator->getLogger());
        out << "unknown exception in IceSSL plug-in";

        // Can't throw from an extern "C" function
        return 0;
    }

    return plugin;
}

//---------------------------------------------------------------------
void MgIceSSL::MgOpenSSLPluginI::configure()
{
    string role = getProperties()->getProperty("MgIceSSL.Role");
    
    if(role == "ClientServer")
    {
        configure(ClientServer);
    }
    else if(role == "Client")
    {
        configure(Client);
    }
    else if(role == "Server")
    {
        configure(Server);
    }
    else
    {
      //throw ???
    }
}

//---------------------------------------------------------------------
void MgIceSSL::MgOpenSSLPluginI::configure(IceSSL::ContextType contextType)
{
  IceUtil::RecMutex::Lock sync(_configMutex);

  Ice::PropertiesPtr props = getProperties();

    if((contextType == Client || contextType == ClientServer))
    {

        // Set general properties of OpenSSL

        IceSSL::GeneralConfig clientGeneral;
	clientGeneral.set("version", "SSLv23");
	clientGeneral.set("cipherlist", "RC4-MD5");
	clientGeneral.set("verifymode", "peer");
	clientGeneral.set("verifydepth", "-1");

	// Set CA path

        IceSSL::CertificateAuthority clientCertAuth;
	string capath = props->getProperty("MgIceSSL.CAPath");
	clientCertAuth.setCAPath(capath);

	// Set RSA certificate

	const int keySize = 2048;  // ??? This info is in the files
	string cert_fname = props->getProperty("MgIceSSL.Client.Certificate");
	string pkey_fname = props->getProperty("MgIceSSL.Client.PrivateKey");
	CertificateFile publicFile(cert_fname, X509_FILETYPE_PEM);
	CertificateFile privateFile(pkey_fname, X509_FILETYPE_PEM);

	CertificateDesc rsaCert(keySize, publicFile, privateFile);

	CertificateDesc dsaCert;
	DiffieHellmanParamsFile dhParams;

        IceSSL::BaseCertificates clientBaseCerts(rsaCert, dsaCert, dhParams);

	initRandSystem(clientGeneral.getRandomBytesFiles());

	_clientContext.configure(clientGeneral, clientCertAuth, clientBaseCerts);
    }

    if((contextType == Server || contextType == ClientServer))
    {

        // Set general properties of OpenSSL

        IceSSL::GeneralConfig serverGeneral;
	serverGeneral.set("version", "SSLv23");
	serverGeneral.set("cipherlist", "RC4-MD5");
	serverGeneral.set("verifymode", "peer");
	serverGeneral.set("verifydepth", "-1");

	// Set CA path

        IceSSL::CertificateAuthority serverCertAuth;
	string capath = props->getProperty("MgIceSSL.CAPath");
	serverCertAuth.setCAPath(capath);

	// Set RSA certificate

	int keySize = 2048;  // ??? This info is in the files
	string cert_fname = props->getProperty("MgIceSSL.Client.Certificate");
	string pkey_fname = props->getProperty("MgIceSSL.Client.PrivateKey");
	CertificateFile publicFile(cert_fname, X509_FILETYPE_PEM);
	CertificateFile privateFile(pkey_fname, X509_FILETYPE_PEM);

	CertificateDesc rsaCert(keySize, publicFile, privateFile);

	CertificateDesc dsaCert;
	DiffieHellmanParamsFile dhParams;

        IceSSL::BaseCertificates serverBaseCerts(rsaCert, dsaCert, dhParams);

	initRandSystem(serverGeneral.getRandomBytesFiles());

	_serverContext.configure(serverGeneral, serverCertAuth, serverBaseCerts);
    }
}

//---------------------------------------------------------------------
void MgIceSSL::MgOpenSSLPluginI::
setCertificateVerifier(IceSSL::ContextType contextType,
		       const IceSSL::CertificateVerifierPtr & verifier)
{
  IceSSL::OpenSSLPluginI::setCertificateVerifier(contextType, verifier);
}
